@@ -7,6 +7,7 @@ from itertools import chain
 import pytest
 
 import click
+from click._utils import UNSET
 
 
 def test_basic_functionality(runner):
@@ -205,19 +206,29 @@ def test_float_option(runner, args, expect):
         assert result.exception is None
 
 
-@pytest.mark.parametrize("default", [True, False])
 @pytest.mark.parametrize(
-    ("args", "expect"), [(["--on"], True), (["--off"], False), ([], None)]
+    ("args", "default", "expect"),
+    [
+        (["--on"], True, True),
+        (["--on"], False, True),
+        (["--on"], None, True),
+        (["--on"], UNSET, True),
+        (["--off"], True, False),
+        (["--off"], False, False),
+        (["--off"], None, False),
+        (["--off"], UNSET, False),
+        ([], True, True),
+        ([], False, False),
+        ([], None, None),
+        ([], UNSET, False),
+    ],
 )
-def test_boolean_switch(runner, default, args, expect):
+def test_boolean_switch(runner, args, default, expect):
     @click.command()
     @click.option("--on/--off", default=default)
     def cli(on):
         return on
 
-    if expect is None:
-        expect = default
-
     result = runner.invoke(cli, args, standalone_mode=False)
     assert result.return_value is expect
 
@@ -229,6 +240,10 @@ def test_boolean_switch(runner, default, args, expect):
         (True, [], True),
         (False, ["--f"], True),
         (False, [], False),
+        # Boolean flags have a 3-states logic.
+        # See: https://github.com/pallets/click/issues/3024#issue-3285556668
+        (None, ["--f"], True),
+        (None, [], None),
     ),
 )
 def test_boolean_flag(runner, default, args, expect):
@@ -261,6 +276,58 @@ def test_boolean_conversion(runner, value, expect):
     assert result.output == expect
 
 
+@pytest.mark.parametrize(
+    ("default", "args", "expected"),
+    # These test cases are similar to the ones in
+    # tests/test_options.py::test_default_dual_option_callback, so keep them in sync.
+    (
+        # Each option is returning its own flag_value, whatever the default is.
+        (True, ["--upper"], "upper"),
+        (True, ["--lower"], "lower"),
+        (False, ["--upper"], "upper"),
+        (False, ["--lower"], "lower"),
+        (None, ["--upper"], "upper"),
+        (None, ["--lower"], "lower"),
+        (UNSET, ["--upper"], "upper"),
+        (UNSET, ["--lower"], "lower"),
+        # Check that the last option wins when both are specified.
+        (True, ["--upper", "--lower"], "lower"),
+        (True, ["--lower", "--upper"], "upper"),
+        # Check that the default is returned as-is when no option is specified.
+        ("upper", [], "upper"),
+        ("lower", [], "lower"),
+        ("uPPer", [], "uPPer"),
+        ("lOwEr", [], "lOwEr"),
+        (" ᕕ( ᐛ )ᕗ ", [], " ᕕ( ᐛ )ᕗ "),
+        (None, [], None),
+        # Default is normalized to None if it is UNSET.
+        (UNSET, [], None),
+        # Special case: if default=True and flag_value is set, the value returned is the
+        # flag_value, not the True Python value itself.
+        (True, [], "upper"),
+        # Non-string defaults are process as strings by the default Parameter's type.
+        (False, [], "False"),
+        (42, [], "42"),
+        (12.3, [], "12.3"),
+    ),
+)
+def test_flag_value_dual_options(runner, default, args, expected):
+    """Check how default is processed when options compete for the same variable name.
+
+    Covers the regression reported in
+    https://github.com/pallets/click/issues/3024#issuecomment-3146199461
+    """
+
+    @click.command()
+    @click.option("--upper", "case", flag_value="upper", default=default)
+    @click.option("--lower", "case", flag_value="lower")
+    def cli(case):
+        click.echo(repr(case), nl=False)
+
+    result = runner.invoke(cli, args)
+    assert result.output == repr(expected)
+
+
 def test_file_option(runner):
     @click.command()
     @click.option("--file", type=click.File("w"))
@@ -475,15 +542,25 @@ def test_choice_argument_none(runner):
     )
     def cli(method: str | None):
         assert isinstance(method, str) or method is None
-        click.echo(method)
+        click.echo(repr(method), nl=False)
 
     result = runner.invoke(cli, ["not-none"])
     assert not result.exception
-    assert result.output == "not-none\n"
+    assert result.output == repr("not-none")
 
-    # None is not yet supported.
     result = runner.invoke(cli, ["none"])
+    assert not result.exception
+    assert result.output == repr(None)
+
+    result = runner.invoke(cli, [])
     assert result.exception
+    assert (
+        "Error: Missing argument '{not-none|none}'. "
+        "Choose from:\n\tnot-none,\n\tnone\n" in result.stderr
+    )
+
+    result = runner.invoke(cli, ["--help"])
+    assert result.output.startswith("Usage: cli [OPTIONS] {not-none|none}\n")
 
 
 def test_datetime_option_default(runner):
